/******************************************************************************
*	(C) Copyright 1986-2002 Rainbow Technologies, Inc. All rights reserved.
*
*	SPROARX.CPP
*
*	Developer : Ajay Kumar Nagariya(S/W Engg) date : 14/12/2001
*
*	Description : Sentinel Superpro Interface for Autocad ARX Applications.
*
*	Function Entry Points:
*				AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg, void * );
*
/******************************************************************************
*	Exported ADS/ARX Functions export(called) by LISP application program
*	sproFormatPacket
*	sproInitialize
*	sproFindFirstUnit
*	sproFindNextUnit
*	sproRead
*	sproExtendedRead
*	sproWrite
*	sproOverwrite
*	sproDecrement
*	sproActivate
*	sproQuery
*	sproGetFullStatus
*	sproGetVersion
*	sproGetKeyInfo
*	sproGetContactServer
*	sproSetContactServer
*	sproGetSubLicense
*	sproReleaseLicense
*	sproGetHardLimit
*	sproEnumServer
*	sproSetProtocol
*	sproSetHeartBeat
******************************************************************************
*                       INCLUDE FILES
******************************************************************************/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <rxdefs.h>             /* ARX definitions */
#include <rxdlinkr.h>			/* ARX linker */
#include <adslib.h>             /* ADS lib definitions */
#include "spromeps.h"           /* SuperPro multi entry point module */

#define ELEMENTS(array) (sizeof(array)/sizeof((array)[0]))

# define MAX_NO_SERVER	10 
# define READ_DATA_LEN	5

//------------- ARX Support Routines ---------------------
static AcRx::AppRetCode funcload    (void);
static AcRx::AppRetCode funcunload  (void);
static AcRx::AppRetCode dofun       (void);
static AcRx::AppRetCode ReturnList  (void);

//------------------- ADS Callable Functions Prototype-----------------------
RB_WORD ADS_FormatPacket		(void);
RB_WORD ADS_Initialize			(void);
RB_WORD ADS_FindFirstUnit		(char *Dev_id);
RB_WORD ADS_FindNextUnit		(void);
RB_WORD ADS_Read					(char *address);
RB_WORD ADS_ExtendedRead		(char *address);
RB_WORD ADS_Write					(char *writePassword,char *address,char *data,
										 unsigned char accessCode);
RB_WORD ADS_Overwrite			(char *writePassword,char *overwritePassword1,
										 char *overwritePassword2,char *address,
										 char *data,unsigned char accessCode);
RB_WORD ADS_Decrement			(char *writePassword , char *address);
RB_WORD ADS_Activate				(char *writePassword,char *activatePassword1,
										 char *activatePassword2,char *address);
RB_WORD ADS_Query					(char *address, char *queryData);
RB_WORD ADS_GetVersion			(void);
RB_WORD ADS_GetFullStatus		(void);
RB_WORD ADS_GetSubLicense		(char *address);
RB_WORD ADS_ReleaseLicense		(char *address,char *numSubLic);
RB_WORD ADS_SetContactServer	(char *serverName);
RB_WORD ADS_GetContactServer	(void);
RB_WORD ADS_SetHeartBeat		(char *heartBeat);
RB_WORD ADS_SetProtocol			(char *protocolFlag);
RB_WORD ADS_GetKeyInfo			(char *keyID, char *keyIndex);
RB_WORD ADS_GetHardLimit		(void);
RB_WORD ADS_EnumServer			(char *enumFlag,char *developerId,char  *serverInfo);

//---------------Global variables

//-------------- ADS functions table for function registration------------------
struct func_entry { char *func_name; int (*func) (); };
static struct func_entry func_table[] = 
			{
				{"sproFormatPacket"    , (int (*) ()) ADS_FormatPacket},
				{"sproInitialize"      , (int (*) ()) ADS_Initialize},
				{"sproFindFirstUnit"   , (int (*) ()) ADS_FindFirstUnit},
				{"sproFindNextUnit"    , (int (*) ()) ADS_FindNextUnit},
				{"sproRead"            , (int (*) ()) ADS_Read},
				{"sproExtendedRead"    , (int (*) ()) ADS_ExtendedRead},
				{"sproWrite"           , (int (*) ()) ADS_Write},
				{"sproOverwrite"       , (int (*) ()) ADS_Overwrite},
				{"sproDecrement"       , (int (*) ()) ADS_Decrement},
				{"sproActivate"        , (int (*) ()) ADS_Activate},
				{"sproQuery"           , (int (*) ()) ADS_Query},
				{"sproGetVersion"      , (int (*) ()) ADS_GetVersion},
				{"sproGetFullStatus"   , (int (*) ()) ADS_GetFullStatus},
				{"sproGetSubLicense"   , (int (*) ()) ADS_GetSubLicense},
				{"sproReleaseLicense"  , (int (*) ()) ADS_ReleaseLicense},
				{"sproSetContactServer", (int (*) ()) ADS_SetContactServer},
				{"sproGetContactServer", (int (*) ()) ADS_GetContactServer},
				{"sproSetHeartBeat",	 (int (*) ()) ADS_SetHeartBeat},
				{"sproSetProtocol", (int (*) ()) ADS_SetProtocol},
				{"sproGetKeyInfo", (int (*) ()) ADS_GetKeyInfo},
				{"sproGetHardLimit", (int (*) ()) ADS_GetHardLimit},
				{"sproEnumServer", (int (*) ()) ADS_EnumServer}
			};


/* AutoCAD ADS stuff */
struct resbuf *res_list;

/* Rainbow Superpro stuff */
RB_WORD	sproResult;
RB_SPRO_APIPACKET  sproApiPacket;  /* SuperPro packet */
RBP_SPRO_APIPACKET packet = &sproApiPacket;

extern "C"
{
	AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg,void * /*ptr*/);
}

/*************************************************************************/
/*         The Main Entry Point                                               */
/*************************************************************************/

AcRx::AppRetCode acrxEntryPoint( AcRx::AppMsgCode msg, void* ptr )
{
	AcRx::AppRetCode status = AcRx::kRetOK;
	switch (msg)
  {
		case AcRx::kInitAppMsg:
			acrxDynamicLinker->unlockApplication(ptr);
			acrxDynamicLinker->registerAppMDIAware(ptr);
			status = AcRx::kRetOK;
		break;

		case AcRx::kInvkSubrMsg:
			dofun();
		break;

		case AcRx::kLoadADSMsg:
			funcload();
		break;

		case AcRx::kUnloadADSMsg:
			funcunload();
		break;

		case AcRx::kUnloadAppMsg:
			status = AcRx::kRetOK;
		break;
		default:
		break;
  }
  return (status);
}

/*************************************************************************/
/*         Load external functions                                       */
/*************************************************************************/

static AcRx::AppRetCode funcload(void)
{
	short int i;
	for (i = 0; i < ELEMENTS(func_table); i++)
		if (ads_defun(func_table[i].func_name, i) ==0 )
			return AcRx::kRetError ;
	ads_printf("Loading...\n");
	return (AcRx::kRetOK);
}

/*************************************************************************/
/*         Unload external functions                                     */
/*************************************************************************/

static AcRx::AppRetCode funcunload(void)
{
	short int i;
	for (i = 0; i <  ELEMENTS(func_table); i++)
		ads_undef(func_table[i].func_name, i);
	
	ads_printf("Unloading...\n");
	return (AcRx::kRetOK);
}

/*************************************************************************/
/*         Execute external functions                                    */
/*************************************************************************/

static AcRx::AppRetCode dofun(void)
{
	struct resbuf *rb;
	int    val ;
	int    sp_accessCode;
	char   *sp_address;
	char   *sp_dev_id;
	char   *sp_writepw,*sp_overwritepw1,*sp_overwritepw2;
	char   *sp_data;
	char   *sp_activatepw1,*sp_activatepw2;
	char   *sp_query;
	char   *sp_contactServer;
	char   *sp_numLic;
	char   *sp_heartBeat;
	char   *sp_protocol;	
	char   *sp_keyid;
	char   *sp_enumFlag;
	char   *sp_DevID;
	char   *sp_numServer;
	char   *sp_keyindex;

	AcRx::AppRetCode sp_ret ;

	val = ads_getfuncode();

	if ( (val < 0) || (val >= ELEMENTS(func_table)) )
	{
		ads_fail("Received nonexistent function code");
		printf("%d", val);
		sp_ret = AcRx::kRetError;
	}

	rb = ads_getargs();   /* get pointer to result buffer */

	switch (val)
	{
		case 0:/*ADS_FormatPacket was defined to have func'code 0.We are not expecting
					any parameter for this function. */
			
			if (rb)
			{
				ads_fail ("This call doesn't need parameters");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_FormatPacket();
			sp_ret = AcRx::kRetOK;
		
		break;

		case 1:/*ADS_Initialize was defined to have func'code 1.We are not expecting
					any parameter for this function. */

			if (rb)
			{
				ads_fail ("This call doesn't need parameters");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_Initialize();
			sp_ret = AcRx::kRetOK;
		
		break;

		case 2:/*ADS_FindFirstUnit was defined to have func'code 2.We are expecting 
					1 parameter (sp_dev_id) for this function.*/
			
			if (!rb)
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for developer ID must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_dev_id = rb->resval.rstring;
			if (rb->rbnext != NULL)
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_FindFirstUnit(sp_dev_id);
			sp_ret = AcRx::kRetOK;
		
		break;

		case 3:/*ADS_FindNextUnit was defined to have func'code 3.We are not expecting
					any parameter for this function. */
			
			if (rb)
			{
				ads_fail ("This call doesn't need parameters");
				sp_ret = AcRx::kRetError;
			}
			ADS_FindNextUnit();
			sp_ret = AcRx::kRetOK;
		
		break;

		case 4:/*ADS_Read was defined to have a func'code of 4. We are expecting 1 
					parameter (sp_address) for this function.*/
			
			if (!rb)
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for address must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_address = rb->resval.rstring;
			if (rb->rbnext != NULL)
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_Read(sp_address);
			sp_ret = AcRx::kRetOK;
		
		break;

		case 5:/*ADS_ExtendedRead was defined to have a func'code of 5.we are expecting
					1 parameter (sp_address) for this function.*/
			
			if (!rb)
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for address must be as string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_address = rb->resval.rstring;
			if (rb->rbnext != NULL)
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_ExtendedRead(sp_address);
			sp_ret = AcRx::kRetOK;
		
		break;

		case 6:/*ADS_Write was defined to have a func'code of 6.We are expecting 
					4 parameters (sp_writepw,sp_address,sp_data,sp_accessCode) 
					for this function.*/
			
			if (!rb)
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for write password must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_writepw = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for address must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_address = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for data must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_data = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSHORT)
			{
				ads_fail("Argument for access code must be a integer.");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_accessCode = rb->resval.rint;
			if (rb->rbnext != NULL)
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_Write(sp_writepw, sp_address, sp_data,(unsigned char) sp_accessCode);
			sp_ret = AcRx::kRetOK;
		
		break;

		case 7:/* ADS_Overwrite was defined to have a func'code of 7.We are expecting 
					6 parameters (sp_writepw,sp_overwritepw1,sp_overwritepw2,sp_address,
					sp_data,sp_accessCode) for this function.*/
			
			if (!rb)
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for write password must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_writepw = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for overwrite password1 must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_overwritepw1 = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for overwrite password2 must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_overwritepw2 = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("argument for address must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_address = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for data must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_data = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSHORT)
			{
				ads_fail("Argument for accesscode must be a integer.");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_accessCode = rb->resval.rint;
			if (rb->rbnext != NULL)
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_Overwrite(sp_writepw,sp_overwritepw1,	sp_overwritepw2,sp_address,
								sp_data, (unsigned char) sp_accessCode);
			sp_ret = AcRx::kRetOK;
			
		break;

		case 8:/* ADS_Decrement was defined to have a func'code of 8.We are expecting
					2 parameter (sp_writepw,sp_address) for this function.*/
			
			if (!rb)
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for write password must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_writepw = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for address must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_address = rb->resval.rstring;
			if (rb->rbnext != NULL)
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_Decrement(sp_writepw, sp_address);
			sp_ret = AcRx::kRetOK;
			
		break;

		case 9:/* ADS_Activate was defined to have a func'code of 9.We are expecting
					4 parameter (sp_writepw,sp_activatepw1,sp_activatepw2,sp_address) 
					for this function.*/
		
			if (!rb)
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
			}
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for write password must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_writepw = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for activate password1 must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_activatepw1 = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for activate password2 must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_activatepw2 = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for address must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_address = rb->resval.rstring;
			if (rb->rbnext != NULL)
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_Activate(sp_writepw, sp_activatepw1,sp_activatepw2, sp_address);
			sp_ret = AcRx::kRetOK;
		break;

		case 10:/* ADS_Query was defined to have a func'code of 10.We are expecting
					2 parameter (sp_address,sp_query) for this function.*/

			if (!rb)
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for address must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_address = rb->resval.rstring;
			if (rb->rbnext == NULL)
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for query data must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_query = rb->resval.rstring;
			if (rb->rbnext != NULL)
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_Query(sp_address, sp_query);
			sp_ret = AcRx::kRetOK;
		
		break;

		case 11 :/* ADS_GetVersion was defined to have a func'code of 11.We are not
						expecting any parameter for this function. */

			if (rb)
			{
				ads_fail ("This call doesn't need parameters");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_GetVersion();
			sp_ret = AcRx::kRetOK;
			
		break;

		case 12 :/* ADS_Status was defined to have a func'code of 12.We are not 
						expecting any parameter for this function. */
			
			if (rb)
			{
				ads_fail ("This call doesn't need parameters");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_GetFullStatus();
			sp_ret = AcRx::kRetOK ;
		
		break;

		case 13 :/* ADS_GetSubLicense was defined to have a func'code of 13.We are
						expecting 1 parameter (sp_address) for this function. */
			
			if (!rb) 
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR) 
			{
				ads_fail("Argument for address must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_address = rb->resval.rstring;
			if (rb->rbnext != NULL) 
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_GetSubLicense(sp_address);
			sp_ret = AcRx::kRetOK;
		
		break;

		case 14 :/* ADS_ReleaseLicense was defined to have a func'code of 14. We are
						expecting 2 parameter (sp_address,sp_numLic) for this function. */
			
			if (!rb) 
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR) 
			{
				ads_fail("Argument for address must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_address = rb->resval.rstring;
			if (rb->rbnext == NULL) 
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR) 
			{
				ads_fail("Argument for numlicense must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_numLic = rb->resval.rstring;
			if (rb->rbnext != NULL) 
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			} 
			ADS_ReleaseLicense(sp_address, sp_numLic);
			sp_ret = AcRx::kRetOK;
			
		break;

		case 15 :/* ADS_SetContactServer was defined to have a func'code of 15.We are
						expecting 1 parameter (sp_address) for this function. */
		
			if (!rb) 
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR) 
			{
				ads_fail("Argument for Server name must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_contactServer = rb->resval.rstring;
			if (rb->rbnext != NULL) 
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_SetContactServer(sp_contactServer);
			sp_ret = AcRx::kRetOK;

		break;

		case 16 :/* ADS_GetContactServer was defined to have a func'code of 16.We are not
						expecting any parameter for this function. */
				
				if (rb)
				{
					ads_fail ("This call doesn't need parameters");
					sp_ret = AcRx::kRetError;
					break;
				}
				ADS_GetContactServer();
				sp_ret = AcRx::kRetOK;

			break;

		case 17 :/* ADS_SetHeartBeat was defined to have a func'code of 17.We are
					expecting 1 parameter (sp_heartBeat) for this function. */
			
			if (!rb) 
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR) 
			{
				ads_fail("Argument for HeartBeat must be string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_heartBeat = rb->resval.rstring;
			if (rb->rbnext != NULL) 
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_SetHeartBeat(sp_heartBeat);
			sp_ret = AcRx::kRetOK;

		break;

		case 18 :/* ADS_SetProtocol was defined to have a func'code of 18.We are
					expecting 1 parameter (sp_protocol) for this function. */
		
			if (!rb) 
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR) 
			{
				ads_fail("Argument for Protocol flag must be :1 for TCP/IP 2 for IPX,4 for NETBIOS and 8 for SAP ");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_protocol = rb->resval.rstring;
			if (rb->rbnext != NULL) 
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_SetProtocol(sp_protocol);
			sp_ret = AcRx::kRetOK;
	
		break;


		case 19:/* ADS_GetKeyInfo was defined to have func'code 19 .We are
					expecting 2 parameter (sp_keyid,sp_next) for this function. */
		
			if (!rb)
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;	 
			}
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for KeyID must be a string");
				sp_ret = AcRx::kRetError;
				break;	 
			}
			sp_keyid = rb->resval.rstring;
			if (rb->rbnext == NULL) 
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;	 
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR) 
			{
				ads_fail("Argument for KeyIndex must be a string");
				sp_ret = AcRx::kRetError;
				break;	 
			}
			sp_keyindex = rb->resval.rstring;
			if (rb->rbnext != NULL)
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;	 
			}
			ADS_GetKeyInfo(sp_keyid,sp_keyindex);
			sp_ret = AcRx::kRetOK;
	
		break;
	
		case 20:/* ADS_GetHardLimit was defined to have func'code 20.We are not 
						expecting any parameter for this function. */
			if (rb)
			{
				ads_fail ("This call doesn't need parameters");
				sp_ret = AcRx::kRetError;
				break;	 
			}
			ADS_GetHardLimit();
			sp_ret = AcRx::kRetOK;

		break;	 

		case 21 :/* ADS_EnumServer was defined to have a func'code of 21.We are
						expecting 3 parameter (sp_enumFlag,sp_DevID,sp_numServer) 
						for this function. */
			if (!rb) 
			{
				ads_fail("Can't get parameter for this call !");
				sp_ret = AcRx::kRetError;
				break;
			}
			if (rb->restype != RTSTR) 
			{
				ads_fail("Argument for EnumServerFlag must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_enumFlag= rb->resval.rstring;
			if (rb->rbnext == NULL) 
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR) 
			{
				ads_fail("Argument for DevID string must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_DevID = rb->resval.rstring;
			if (rb->rbnext == NULL) 
			{
				ads_fail("Too few arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			rb = rb->rbnext;
			if (rb->restype != RTSTR)
			{
				ads_fail("Argument for NumServerInfo must be a string");
				sp_ret = AcRx::kRetError;
				break;
			}
			sp_numServer = rb->resval.rstring;
			if (rb->rbnext != NULL)
			{
				ads_fail("Too many arguments");
				sp_ret = AcRx::kRetError;
				break;
			}
			ADS_EnumServer(sp_enumFlag,sp_DevID,sp_numServer);
			sp_ret = AcRx::kRetOK;
			break;

		break;
	
			default:
				ads_fail("Received nonexistent function code");
				sp_ret = AcRx::kRetError;
				break;
			
			break;
	
	} /* end switch */
	return (sp_ret);
}


/****************************************************************************
 *
 *               Implementation of the external functions
 *
 *  Purpose : Every function is an ADS bridge from the AUTOLisp Call
 *            to the Sentinel routines
 *
 ****************************************************************************/

RB_WORD ADS_FormatPacket(void)
{
	// SSP Clinet Lib API call
	sproResult = RNBOsproFormatPacket(packet, sizeof(RB_SPRO_APIPACKET));

	res_list = ads_buildlist(RTSHORT, (short)sproResult, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_Initialize(void)
{
	// SSP Clinet Lib API call
	sproResult = RNBOsproInitialize(packet);

	res_list = ads_buildlist(RTSHORT, (short)sproResult, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_FindFirstUnit(char *Dev_id)
{
   RB_WORD developerID ;
   sscanf(Dev_id, "%x", &developerID);

	// SSP Clinet Lib API call
   sproResult = RNBOsproFindFirstUnit(packet, developerID);
   
	res_list = ads_buildlist(RTSHORT, (short)sproResult, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_FindNextUnit(void)
{
   // SSP Clinet Lib API call
	sproResult = RNBOsproFindNextUnit(packet);

   res_list = ads_buildlist(RTSHORT, (short)sproResult, RTNONE);
   return (ReturnList());
}


RB_WORD ADS_Read(char *address)
{
   RB_WORD InAddress, MemoryContents;
   char ReturnMemoryContents[READ_DATA_LEN];
   sscanf(address, "%x", &InAddress);
   
	// SSP Clinet Lib API call
	sproResult = RNBOsproRead(packet, InAddress, &MemoryContents);
   
	_itoa((int) MemoryContents, ReturnMemoryContents, 16);
   _strupr(ReturnMemoryContents);
   res_list = ads_buildlist(RTSHORT, (short) sproResult,RTSTR, 
									 ReturnMemoryContents,RTNONE);
   return (ReturnList());
}


RB_WORD ADS_ExtendedRead(char *address)
{

   RB_WORD InAddress, MemoryContents;
   RB_BYTE AccessCode;
   char RetMemoryContents[READ_DATA_LEN];
   sscanf(address, "%x", &InAddress);
   
	// SSP Clinet Lib API call
	sproResult = RNBOsproExtendedRead(packet, InAddress,&MemoryContents, &AccessCode);
   
	sprintf(RetMemoryContents, "%04X", MemoryContents);
   _strupr(RetMemoryContents);
   res_list = ads_buildlist(RTSHORT, (short) sproResult,RTSTR, 
									 RetMemoryContents,RTSHORT, (short) AccessCode, RTNONE);
   	return (ReturnList());
}


RB_WORD ADS_Write(char *writePassword, char *address,
char *data, unsigned char accessCode)
{
   RB_WORD InWritepw, InAddress, InData;
   sscanf(writePassword, "%x", &InWritepw);
   sscanf(address, "%x", &InAddress);
   sscanf(data, "%x", &InData);
   
	// SSP Clinet Lib API call
	sproResult = RNBOsproWrite(packet,  InWritepw, InAddress,InData, accessCode);
	
   res_list = ads_buildlist(RTSHORT, (short)sproResult, RTNONE);
   return (ReturnList());
}


RB_WORD ADS_Overwrite(char *writePassword, char *overwritePassword1,
char *overwritePassword2, char *address, char *data,
unsigned char accessCode)
{
	unsigned int InWritepw, InOverwritepw1, InOverwritepw2, InAddress, InData;
	/* conversion of the hex input strings to integers */
	sscanf(writePassword, "%x", &InWritepw);
	sscanf(overwritePassword1, "%x", &InOverwritepw1);
	sscanf(overwritePassword2, "%x", &InOverwritepw2);
	sscanf(address, "%x", &InAddress);
	sscanf(data, "%x", &InData);

	// SSP Clinet Lib API call
	sproResult = RNBOsproOverwrite( packet,InWritepw,InOverwritepw1,InOverwritepw2,
											  InAddress,InData,accessCode);

	res_list = ads_buildlist(RTSHORT, (short)sproResult, RTNONE);
	return (ReturnList());
}



RB_WORD ADS_Decrement(char *writePassword, char *address)
{
	unsigned int InWritepw, InAddress;
	sscanf(writePassword, "%x", &InWritepw);
	sscanf(address, "%x", &InAddress);

	// SSP Clinet Lib API call
	sproResult = RNBOsproDecrement(packet, InWritepw, InAddress);

	res_list = ads_buildlist(RTSHORT, (short)sproResult, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_Activate(char *writePassword, char *activatePassword1,
							char *activatePassword2, char *address)
{
	unsigned int InWritepw, InActivatepw1, InActivatepw2, InAddress;
	sscanf(writePassword, "%x", &InWritepw);
	sscanf(activatePassword1, "%x", &InActivatepw1);
	sscanf(activatePassword2, "%x", &InActivatepw2);
	sscanf(address, "%x", &InAddress);

	// SSP Clinet Lib API call
	sproResult=RNBOsproActivate( packet,InWritepw,InActivatepw1,InActivatepw2,InAddress);

	res_list = ads_buildlist(RTSHORT, (short)sproResult, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_Query(char *address, char *queryData)
{
	RB_WORD InAddress;
	RB_WORD i = 0;
	RB_DWORD response32;
	unsigned char *response, *response1;
	int HEX;
	int j, str_ok;
	unsigned int k ;
	char *cp;
	unsigned char hexstr[128] ;
	char *ch1;
	sscanf(address, "%x", &InAddress);
	response = (unsigned char *)malloc(132);
	response1 = (unsigned char *)malloc(132);
	strcpy((char *)response, "\0");
	strcpy((char *)response1, "\0");
	/* the input string is converted */
	HEX = TRUE;
	cp = queryData;
	
	while (*cp) 
	{
		if (!isxdigit(*cp)) 
		{
			HEX = FALSE;
		break;
		}
		cp++;
	}

	if (!HEX) 
	{
		printf("Error !!! - Query data must be in hex. The length of data must be even and between 8 to 112.\n");
		str_ok = FALSE ;
	} 
	else if ((strlen(queryData) < 8) ||( strlen(queryData) > 112)) 
	{
		printf("Error !!! - Query data must be in hex. The length of data must be even and between 8 to 112.\n");
		str_ok = FALSE ;
	} 
	else 
	{
		queryData[strlen(queryData)+1] = '\0' ; /** add another 0 for char2hex convr **/
		for (i=j=0; i <= strlen(queryData)+1 ; j++) 
		{
			sscanf( &queryData[i], "%2hx", (int *)&(hexstr[j]) ) ;
			i = i + 2 ;
		}
		for( i=0 ; i<j ; i++ ) 
		{
			queryData[i] = hexstr[i] ;
		}
		i-- ;
	}

	// SSP Clinet Lib API call
	sproResult=RNBOsproQuery(packet,InAddress,queryData,(RBP_VOID)response,&response32,i);

	/* the response string is converted back to a Hexstring */
	memset(response1, 0, 132);
	ch1 = (char *)response1;
	for (j=0 ; j < i; j++) 
	{
		k = (unsigned int) response[j];
		sprintf(ch1, "%02hX", k);
		ch1 += 2;
	}
	_strupr((char *)response1);
	res_list = ads_buildlist(RTSHORT, (short)sproResult, RTSTR,response1, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_GetVersion(void)
{
	RB_BYTE majVer , minVer , rev , osDrvrType ;

	char *DriverType ;
	// SSP Clinet Lib API call
	sproResult = RNBOsproGetVersion (packet , &majVer , &minVer , &rev , &osDrvrType);
	
	if (sproResult == 0)
	{
		switch(osDrvrType)
		{
		
			case 5:
				DriverType = "5(Windows NT System Driver)";
				break;
			case 7:
				DriverType = "7(Windows 95 System Driver)";
				break;
			case 8:
				DriverType = "8(Novel System Drive)";
				break;
				default:
					DriverType = "10(Unknown Driver Type)";
                      
                       break;
		
			}
	
		res_list = ads_buildlist(RTSHORT,(short) sproResult,RTSHORT, (short) majVer ,RTSHORT ,
				(short) minVer,RTSHORT, (short) rev , RTSTR ,(char *) DriverType , RTNONE);
	}
	else
		res_list = ads_buildlist(RTSHORT,(short) sproResult,RTNONE);
	return (ReturnList());
}


RB_WORD ADS_GetFullStatus (void)
{
	// SSP Clinet Lib API call
	sproResult = RNBOsproGetFullStatus (packet);

	res_list = ads_buildlist (RTSHORT , (short) sproResult , RTNONE);
	return(ReturnList());
}


RB_WORD ADS_GetSubLicense(char *address)
{
	RB_WORD InAddress;
	sscanf(address, "%x", &InAddress);

	// SSP Clinet Lib API call
	sproResult = RNBOsproGetSubLicense(packet, InAddress);

	res_list = ads_buildlist(RTSHORT, (short) sproResult, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_ReleaseLicense(char *address, char *numLic)
{
	RB_WORD InAddress;
	RB_WORD  InNumLic; 
	sscanf(address, "%x", &InAddress);
	sscanf(numLic, "%d", &InNumLic);

	// SSP Clinet Lib API call
	sproResult = RNBOsproReleaseLicense(packet, InAddress, &InNumLic);

	res_list =ads_buildlist(RTSHORT, (short)sproResult, RTSHORT, (short)InNumLic, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_SetContactServer(char *contactServer)
{

	// SSP Clinet Lib API call
	sproResult = RNBOsproSetContactServer(packet,(char *) contactServer);

	res_list = ads_buildlist(RTSHORT, (short) sproResult, RTNONE);
	return (ReturnList());
}



RB_WORD ADS_SetHeartBeat (char *heartBeat)
{

	long hBeat;
	sscanf(heartBeat, "%ld", &hBeat);	

	// SSP Clinet Lib API call
	sproResult = RNBOsproSetHeartBeat(packet,hBeat);

	res_list = ads_buildlist(RTSHORT, (short) sproResult, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_SetProtocol(char *protocolFlag)
{
	unsigned int pFlag ;
	sscanf(protocolFlag, "%x", &pFlag);	

	// SSP Clinet Lib API call
	sproResult = RNBOsproSetProtocol(packet,pFlag);
	
	res_list = ads_buildlist(RTSHORT, (short) sproResult, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_GetKeyInfo(char *keyID,char *keyIndex)

{
	RB_WORD Key,Index;
	sscanf(keyID, "%x", &Key);	
	sscanf(keyIndex, "%d", &Index);	
	char hexdevid[5];
	NSPRO_MONITOR_INFO  nsproMonitorInfo1;
	
	// SSP Clinet Lib API call
	sproResult = RNBOsproGetKeyInfo (packet,Key, Index,&nsproMonitorInfo1);
	// get key info seperately from structure
	
	RB_WORD hardlimit= nsproMonitorInfo1.sproKeyMonitorInfo.hardLimit;
	RB_WORD licuse= nsproMonitorInfo1.sproKeyMonitorInfo.inUse;
	RB_WORD devid= nsproMonitorInfo1.sproKeyMonitorInfo.devId;
	RB_WORD lictimeout =nsproMonitorInfo1.sproKeyMonitorInfo.numTimeOut;
	RB_WORD highuse=nsproMonitorInfo1.sproKeyMonitorInfo.highestUse;
	sprintf(hexdevid, "%04X", devid);

	res_list = ads_buildlist(RTSHORT, (short) sproResult,RTSTR,  hexdevid,RTLONG,(int) hardlimit,RTSHORT,(short) licuse,RTSHORT, 
									(short) lictimeout,RTSHORT, (short) highuse, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_GetHardLimit(void)
{
	RB_WORD hardLimit;

	// SSP Clinet Lib API call
	sproResult = RNBOsproGetHardLimit(packet,&hardLimit);

	res_list = ads_buildlist(RTSHORT, (short) sproResult,RTLONG,(int)hardLimit ,RTNONE);
	return (ReturnList());
}


RB_WORD ADS_GetContactServer(void)
{
	char ContactServer[64];
	memset(ContactServer,'\0',64);

	// SSP Clinet Lib API call
	sproResult = RNBOsproGetContactServer(packet, ContactServer, 64);

	res_list = ads_buildlist(RTSHORT, (short)sproResult, RTSTR,ContactServer, RTNONE);
	return (ReturnList());
}


RB_WORD ADS_EnumServer(char *eFlag,char *developerId,char  *serverInfo)
{
	ENUM_SERVER_FLAG enumFlag;

	NSPRO_SERVER_INFO  srvrInfo[MAX_NO_SERVER];

	RB_WORD DevID,numserver ;
	sscanf(eFlag, "%x", &enumFlag);	   
	sscanf(developerId, "%x", &DevID);	
	sscanf(serverInfo, "%d", &numserver);	

	switch(enumFlag)
	{
		
		case 1:
			enumFlag = NSPRO_RET_ON_FIRST;
			break;

		case 2:
			enumFlag = NSPRO_GET_ALL_SERVERS;
			break;
	
		case 4:
			enumFlag = NSPRO_RET_ON_FIRST_AVAILABLE;
			break;
	
	}
	
	// SSP Clinet Lib API call
	sproResult = RNBOsproEnumServer(enumFlag, DevID, srvrInfo,&numserver);

	switch(numserver)
	{
		case 1:
			res_list = ads_buildlist(RTSHORT,(short)sproResult,RTSHORT,(short)numserver,RTSTR,
			srvrInfo[0].serverAddress,RTLONG,(int)srvrInfo[0].numLicAvail,RTNONE);		   
			break;
		
		case 2:
			res_list = ads_buildlist(RTSHORT,(short)sproResult,RTSHORT,(short)numserver,RTSTR,
			srvrInfo[0].serverAddress,RTLONG,(int)srvrInfo[0].numLicAvail,RTSTR,
			srvrInfo[1].serverAddress,RTLONG,(int)srvrInfo[1].numLicAvail,RTNONE);		   
			break;
		
		case 3:
			res_list = ads_buildlist(RTSHORT,(short)sproResult,RTSHORT,(short)numserver,RTSTR,
			srvrInfo[0].serverAddress,RTLONG,(int)srvrInfo[0].numLicAvail,RTSTR,
			srvrInfo[1].serverAddress,RTLONG,(int)srvrInfo[1].numLicAvail,RTSTR,
			srvrInfo[2].serverAddress,RTLONG,(int)srvrInfo[2].numLicAvail,RTNONE);		   
			break;
		
		case 4:
			res_list = ads_buildlist(RTSHORT,(short)sproResult,RTSHORT,(short)numserver,RTSTR,
			srvrInfo[0].serverAddress,RTLONG,(int)srvrInfo[0].numLicAvail,RTSTR,
			srvrInfo[1].serverAddress,RTLONG,(int)srvrInfo[1].numLicAvail,RTSTR,
			srvrInfo[2].serverAddress,RTLONG,(int)srvrInfo[2].numLicAvail,RTSTR,
			srvrInfo[3].serverAddress,RTLONG,(int)srvrInfo[3].numLicAvail,RTNONE);		   	   
		  break;
		
		case 5:
			res_list = ads_buildlist(RTSHORT,(short)sproResult,RTSHORT,(short)numserver,RTSTR,
			srvrInfo[0].serverAddress,RTLONG,(int)srvrInfo[0].numLicAvail,RTSTR,
			srvrInfo[1].serverAddress,RTLONG,(int)srvrInfo[1].numLicAvail,RTSTR,
			srvrInfo[2].serverAddress,RTLONG,(int)srvrInfo[2].numLicAvail,RTSTR,
			srvrInfo[3].serverAddress,RTLONG,(int)srvrInfo[3].numLicAvail,RTSTR,
			srvrInfo[4].serverAddress,RTLONG,(int)srvrInfo[4].numLicAvail,RTNONE);		   	   
		  break;
		
		case 6:
			res_list = ads_buildlist(RTSHORT,(short)sproResult,RTSHORT,(short)numserver,RTSTR,
			srvrInfo[0].serverAddress,RTLONG,(int)srvrInfo[0].numLicAvail,RTSTR,
			srvrInfo[1].serverAddress,RTLONG,(int)srvrInfo[1].numLicAvail,RTSTR,
			srvrInfo[2].serverAddress,RTLONG,(int)srvrInfo[2].numLicAvail,RTSTR,
			srvrInfo[3].serverAddress,RTLONG,(int)srvrInfo[3].numLicAvail,RTSTR,
			srvrInfo[4].serverAddress,RTLONG,(int)srvrInfo[4].numLicAvail,RTSTR,
			srvrInfo[5].serverAddress,RTLONG,(int)srvrInfo[5].numLicAvail,RTNONE);		   	   
		  break;
		
		case 7:
			res_list = ads_buildlist(RTSHORT,(short)sproResult,RTSHORT,(short)numserver,RTSTR,
			srvrInfo[0].serverAddress,RTLONG,(int)srvrInfo[0].numLicAvail,RTSTR,
			srvrInfo[1].serverAddress,RTLONG,(int)srvrInfo[1].numLicAvail,RTSTR,
			srvrInfo[2].serverAddress,RTLONG,(int)srvrInfo[2].numLicAvail,RTSTR,
			srvrInfo[3].serverAddress,RTLONG,(int)srvrInfo[3].numLicAvail,RTSTR,
			srvrInfo[4].serverAddress,RTLONG,(int)srvrInfo[4].numLicAvail,RTSTR,
			srvrInfo[5].serverAddress,RTLONG,(int)srvrInfo[5].numLicAvail,RTSTR,
			srvrInfo[6].serverAddress,RTLONG,(int)srvrInfo[6].numLicAvail,RTNONE);		   	   
		  break;
		
		case 8:
			res_list = ads_buildlist(RTSHORT,(short)sproResult,RTSHORT,(short)numserver,RTSTR,
			srvrInfo[0].serverAddress,RTLONG,(int)srvrInfo[0].numLicAvail,RTSTR,
			srvrInfo[1].serverAddress,RTLONG,(int)srvrInfo[1].numLicAvail,RTSTR,
			srvrInfo[2].serverAddress,RTLONG,(int)srvrInfo[2].numLicAvail,RTSTR,
			srvrInfo[3].serverAddress,RTLONG,(int)srvrInfo[3].numLicAvail,RTSTR,
			srvrInfo[4].serverAddress,RTLONG,(int)srvrInfo[4].numLicAvail,RTSTR,
			srvrInfo[5].serverAddress,RTLONG,(int)srvrInfo[5].numLicAvail,RTSTR,
			srvrInfo[6].serverAddress,RTLONG,(int)srvrInfo[6].numLicAvail,RTSTR,
			srvrInfo[7].serverAddress,RTLONG,(int)srvrInfo[7].numLicAvail,RTNONE);		   	   
			break;
		
		case 9:
			res_list = ads_buildlist(RTSHORT,(short)sproResult,RTSHORT,(short)numserver,RTSTR,
			srvrInfo[0].serverAddress,RTLONG,(int)srvrInfo[0].numLicAvail,RTSTR,
			srvrInfo[1].serverAddress,RTLONG,(int)srvrInfo[1].numLicAvail,RTSTR,
			srvrInfo[2].serverAddress,RTLONG,(int)srvrInfo[2].numLicAvail,RTSTR,
			srvrInfo[3].serverAddress,RTLONG,(int)srvrInfo[3].numLicAvail,RTSTR,
			srvrInfo[4].serverAddress,RTLONG,(int)srvrInfo[4].numLicAvail,RTSTR,
			srvrInfo[5].serverAddress,RTLONG,(int)srvrInfo[5].numLicAvail,RTSTR,
			srvrInfo[6].serverAddress,RTLONG,(int)srvrInfo[6].numLicAvail,RTSTR,
			srvrInfo[7].serverAddress,RTLONG,(int)srvrInfo[7].numLicAvail,RTSTR,
			srvrInfo[8].serverAddress,RTLONG,(int)srvrInfo[8].numLicAvail,RTNONE);		   	   
		  break;
		
		case 10:
			res_list = ads_buildlist(RTSHORT,(short)sproResult,RTSHORT,(short)numserver,RTSTR,
			srvrInfo[0].serverAddress,RTLONG,(int)srvrInfo[0].numLicAvail,RTSTR,
			srvrInfo[1].serverAddress,RTLONG,(int)srvrInfo[1].numLicAvail,RTSTR,
			srvrInfo[2].serverAddress,RTLONG,(int)srvrInfo[2].numLicAvail,RTSTR,
			srvrInfo[3].serverAddress,RTLONG,(int)srvrInfo[3].numLicAvail,RTSTR,
			srvrInfo[4].serverAddress,RTLONG,(int)srvrInfo[4].numLicAvail,RTSTR,
			srvrInfo[5].serverAddress,RTLONG,(int)srvrInfo[5].numLicAvail,RTSTR,
			srvrInfo[6].serverAddress,RTLONG,(int)srvrInfo[6].numLicAvail,RTSTR,
			srvrInfo[7].serverAddress,RTLONG,(int)srvrInfo[7].numLicAvail,RTSTR,
			srvrInfo[8].serverAddress,RTLONG,(int)srvrInfo[8].numLicAvail,RTSTR,
			srvrInfo[9].serverAddress,RTLONG,(int)srvrInfo[9].numLicAvail,RTNONE);		   	   
		  break;
		
		default :
			res_list = ads_buildlist(RTSHORT,(short)sproResult,RTSHORT,(short)numserver,RTNONE);		   
		  break;

	   }
	return (ReturnList());
	
}



/******************************************************************************
*
*  function to return resbuf data to AutoLisp, then free up space
*
*******************************************************************************/
static AcRx::AppRetCode ReturnList(void)
{
	if (res_list == NULL)
	{
		ads_fail("Couldn't create list\n");
		return(AcRx::kRetError);
	}
	ads_retlist(res_list);
	ads_relrb(res_list);
//	free();
	return(AcRx::kRetOK);
} /* ReturnList */
